#ifndef ImageRecordList_h
#define ImageRecordList_h

// $Id: ImageRecordList.h,v 1.13 2013/10/25 09:39:36 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "Debug.h"
#include "File.h"
#include "ImageRecord.h"
#include "TimeStamp.h"
#include "ValidFileThread.h"

#include <QObject>
#include <QList>

//! handles list of files saved into resource file for later reopening
class ImageRecordList: public QObject, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    ImageRecordList( QObject* );

    //! destructor
    virtual ~ImageRecordList( void )
    { Debug::Throw( "ImageRecordList::~ImageRecordList.\n" ); }

    //! returns true if file is found in list
    virtual bool contains( const ImageRecord& record ) const
    { return contains( record.file().file(), record.version() ); }

    //! returns true if file is found in list
    virtual bool contains( const File&, int = -1 ) const;

    //! add file.
    virtual ImageRecord& add( const File& file )
    { return _add( ImageRecord( file ), false ); }

    //! add file.
    virtual ImageRecord& add( const ImageRecord& record )
    { return _add( record ); }

    //! remove file from database
    virtual void remove( const File&, int = -1 );

    //! get filerecord associated to a name
    /*! creates new fileRecord if not found */
    virtual ImageRecord& get( const File& file, int version = 0 )
    { return _add( ImageRecord( file, version ), false ); }

    //! gets file list size
    virtual int size( void ) const
    { return _records().size(); }

    //! all records
    ImageRecord::List records( void ) const
    { return _truncatedList( _records() ); }

    //! set record
    virtual void set( const ImageRecord::List& );

    //! all files
    QList< File > files( void ) const;

    //! get last valid file
    virtual ImageRecord lastValidFile( void );

    //! returns true if file list can be cleaned
    virtual bool cleanEnabled( void ) const
    { return (check()) ? cleanEnabled_ : !_records().empty(); }

    //! clean files. Remove either invalid or all files, depending on check_
    virtual void clean( void );

    //! clear files. Remove all
    virtual void clear( void );

    //! check flag
    virtual const bool& check( void ) const
    { return check_; }

    //! check_ flag
    virtual void setCheck( const bool& value )
    { check_ = value; }

    //! Read fileList from file
    virtual bool read( File = File() );

    //! write fileList to file
    virtual bool write( File = File() );

    Q_SIGNALS:

    //! emmited when thread has completed validity check
    void validFilesChecked( void );

    //! emmited when contents is changed
    void contentsChanged( void );

    public Q_SLOTS:

    //! run thread to check file validity
    void checkValidFiles( void );

    protected:

    //! set db file
    virtual bool _setDBFile( const File& file );

    //! maximum Size
    virtual void _setMaxSize( const int& value );

    //! maximum size
    virtual const int& _maxSize( void ) const
    { return maxSize_; }

    //! add record to current list
    virtual ImageRecord& _add(
        const ImageRecord& record,
        const bool& update_timestamp = true,
        const bool& emit_signal = true );

    //! truncate list if larger than maxSize_
    virtual ImageRecord::List _truncatedList( ImageRecord::List ) const;

    //! list of files records
    virtual const ImageRecord::List& _records( void ) const
    { return records_; }

    //! list of files records
    virtual FileRecord::Set _fileRecords( void ) const
    {
        FileRecord::Set out;
        foreach( const ImageRecord& record, records_ )
        { out << record.file(); }
        return out;
    }

    //! list of files records
    virtual ImageRecord::List& _records( void )
    { return records_; }

    protected Q_SLOTS:

    //! process records from threads
    void _processRecords( const FileRecord::List&, bool );

    private Q_SLOTS:

    //! update configuration
    void _updateConfiguration( void );

    //! save configuration
    void _saveConfiguration( void );

    private:

    //! clean enabled
    void _setCleanEnabled( const bool& value )
    { cleanEnabled_ = value; }

    //! maximum size (negative means no limit)
    int maxSize_;

    //! if true, check file validity
    bool check_;

    //! true if clean action is enabled
    bool cleanEnabled_;

    //! thread to check file validity
    ValidFileThread thread_;

    //! current list of files
    ImageRecord::List records_;

    //! file from/to wich the files are saved
    File dbFile_;

};
#endif
