#ifndef MainWindow_h
#define MainWindow_h

// $Id: MainWindow.h,v 1.40 2013/10/25 09:39:36 hupereir Exp $
/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "BaseMainWindow.h"
#include "Counter.h"
#include "ImageRecord.h"

#include <QBasicTimer>
#include <QTimerEvent>
#include <QLabel>
#include <QAction>
#include <QScrollArea>
#include <QUndoStack>

// forward declaration
class Menu;
class RefPointPanel;
class DisplayWidget;
class NotificationWidget;
class RefPointListWidget;
class BaseStatusBar;

class MainWindow: public BaseMainWindow, public Counter
{

    Q_OBJECT

    public:

    //! constructor
    MainWindow( QWidget* = 0 );

    //! destructor
    virtual ~MainWindow( void )
    {}

    //! image record
    void setImageRecord( const ImageRecord& );

    //! image record
    bool hasImageRecord( void ) const;

    //! image record
    ImageRecord imageRecord( void );

    //! display
    DisplayWidget& displayWidget( void ) const
    { return *displayWidget_; }

    //! reference point panel
    RefPointPanel& refPointPanel( void ) const
    { return *refPointPanel_; }

    //! list widget
    RefPointListWidget& refPointListWidget( void ) const
    { return *refPointListWidget_; }

    //! clear
    void clear( void );

    //!@name actions
    //@{

    //! default errors
    QAction& defaultErrorsAction( void ) const
    { return *defaultErrorsAction_; }

    //! default systematic errors
    QAction& defaultErrorsSystAction( void ) const
    { return *defaultErrorsSystAction_; }

    //! undo action
    QAction& undoAction( void ) const
    { return *undoAction_; }

    //! redo action
    QAction& redoAction( void ) const
    { return *redoAction_; }

    //@}

    //!@name undo/redo stack
    //@{

    //! add point
    void updateReferencePoint( int, const RefPoint& );

    //! remove point
    void removeReferencePoint( int );

    //! add point
    void addPoint( int, const RefPoint& );

    //! add point
    void insertPoint( int, int, const RefPoint& );

    //! remove point
    void removePoint( int, const RefPoint& );

    //! replace point
    void replacePoint( int, const RefPoint&, const RefPoint& );

    //@}

    protected:

    //! timer event
    void timerEvent( QTimerEvent* );

    protected Q_SLOTS:

    //! pointer position changed
    void _pointerMoved( QPoint, RefPoint::HoverMode );

    //! display widget left
    void _displayWidgetEntered( void );

    //! display widget left
    void _displayWidgetLeft( void );

    //! edit reference point
    void _editRefPoint( QAction* );

    //! edit reference point
    void _editRefPoint( int );

    //! clear reference point
    void _clearRefPoint( QAction* );

    //! clear reference point
    void _clearRefPoint( int );

    //! update ref points when changed
    void _updateRefPoints( void );

    //! update points
    void _updatePoints( void );

    //! update selected points
    void _updateSelection( void );

    //! update selected points
    void _updateRefPointSelection( void );

    //! select point
    void _selectPoint( void );

    //! add point
    void _addPoint( QPointF );

    //! remove point
    void _removePoint( void );

    //! remove points
    void _removePoints( const RefPoint::List& );

    //! move points up
    void _movePointsUp( const RefPoint::List& );

    //! move points up
    void _movePointsDown( const RefPoint::List& );

    //! copy
    void _copyPoint( void );

    //! paste
    void _pastePoint( QPointF );

    //! edit errors
    void _editErrors( void );

    //! edit errors
    void _addDefaultErrors( void );

    //! edit errors
    void _addDefaultErrorsSyst( void );

    //! edit errors
    void _editErrors( RefPoint );

    //! edit errors
    void _addDefaultErrors( RefPoint );

    //! edit errors
    void _addDefaultErrorsSyst( RefPoint );

    //! edit errors
    void _addDefaultErrorsToAll( void );

    //! edit errors
    void _addDefaultErrorsSystToAll( void );

    //! drag point
    void _dragPoint( QPointF, RefPoint::DragMode );

    //! move point(s) using arrow keys
    void _movePointsUp( void ) { return _movePoints( Qt::Key_Up ); }
    void _movePointsDown( void ) { return _movePoints( Qt::Key_Down ); }
    void _movePointsLeft( void ) { return _movePoints( Qt::Key_Left ); }
    void _movePointsRight( void ) { return _movePoints( Qt::Key_Right ); }

    //! regularize points
    void _regularize( void );

    protected:

    //! notification widget
    NotificationWidget& _notificationWidget( void ) const
    { return *notificationWidget_; }

    //! move point(s) using arrow keys
    void _movePoints( Qt::Key );

    private Q_SLOTS:

    //! splitter moved
    void _splitterMoved( void );

    //! configuration
    void _updateConfiguration( void );

    private:

    //! install actions
    void _installActions( void );

    //! image record
    ImageRecord record_;

    //! menu
    Menu* menu_;

    //! local widget to allow size adjustment
    class Widget: public QWidget, public Counter
    {

        public:

        //! constructor
        Widget( QWidget* parent = 0 ):
            QWidget( parent ),
            Counter( "MainWindow::Counter" ),
            defaultWidth_(0),
            defaultHeight_(0)
        {}

        //! destructor
        virtual ~Widget( void )
        {}

        //! default size
        void setDefaultWidth( int );

        //! default size
        void setDefaultHeight( int );

        //! default width
        const int& defaultWidth( void ) const
        { return defaultWidth_; }

        //! default width
        const int& defaultHeight( void ) const
        { return defaultHeight_; }

        //! size
        QSize sizeHint( void ) const;

        private:

        //! default width;
        int defaultWidth_;

        //! default height
        int defaultHeight_;

    };

    //! containers
    Widget* listContainer_;

    //! scroll area
    QScrollArea* scrollArea_;

    //! ref point panel
    RefPointPanel* refPointPanel_;

    //! display widget
    DisplayWidget* displayWidget_;

    //! notification widget
    NotificationWidget* notificationWidget_;

    //! ref point list
    RefPointListWidget* refPointListWidget_;

    //! file display lineEdit
    QLabel* fileEditor_;

    //! status bar
    BaseStatusBar* statusbar_;

    //! undo stack
    QUndoStack* undoStack_;

    //!@name actions
    //@{

    //! default errors action
    QAction* defaultErrorsAction_;

    //! default errors action
    QAction* defaultErrorsSystAction_;

    //! undo action
    QAction* undoAction_;

    //! redo action
    QAction* redoAction_;

    //@}

    //! true if some point was hovered
    bool wasHovered_;

    //! resize timer
    /*! needed to store Keyword list width */
    QBasicTimer resizeTimer_;

    //! local string format
    QString localFormat_;

};

#endif
