// $Id: RefPoint.cpp,v 1.19 2014/05/05 15:03:26 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "RefPoint.h"
#include "Debug.h"
#include "XmlDef.h"

//__________________________________________________________________
RefPoint::PointF::PointF( const QDomElement& element ):
    Counter( "RefPoint::PointF" )
{

    Debug::Throw( "RefPoint::PointF::PointF (dom)\n" );

    QDomNamedNodeMap attributes( element.attributes() );
    for( int i=0; i<attributes.count(); i++ )
    {
        QDomAttr attribute( attributes.item( i ).toAttr() );
        if( attribute.isNull() ) continue;

        QString name( attribute.name() );
        QString value( attribute.value() );
        if( name == Xml::X ) setX( value.toDouble() );
        else if( name == Xml::Y ) setY( value.toDouble() );

    }
    
}

//__________________________________________________________________
QDomElement RefPoint::PointF::domElement( const QString& name, QDomDocument& parent ) const
{

    Debug::Throw() << "RefPoint::PointF::domElement - name: " << name << endl;

    QDomElement out = parent.createElement( name );
    out.setAttribute( Xml::X, QString::number(x()) );
    out.setAttribute( Xml::Y, QString::number(y()) );
    return out;
}

//__________________________________________________________________
QString RefPoint::PointF::toString( const QString& format ) const
{
    if( format.isEmpty() )
    {
        QString buffer;
        QTextStream( &buffer ) << "(" << x() << "," << y() << ")";
        return buffer;
    } else {
        QString buffer;
        QTextStream( &buffer )
            << "("
            << QString().sprintf( format.toLatin1().constData(), x() ) << ","
            << QString().sprintf( format.toLatin1().constData(), y() ) << ")";
        return buffer;
    }

}

//__________________________________________________________________
RefPoint::Error::Error( const QDomElement& element ):
    Counter( "RefPoint::Error" ),
    mode_( NoError ),
    plus_( 0 ),
    minus_( 0 ),
    relative_( false )
{

    Debug::Throw( "RefPoint::Error::Error (dom).\n" );

    QDomNamedNodeMap attributes( element.attributes() );
    for( int i=0; i<attributes.count(); i++ )
    {
        QDomAttr attribute( attributes.item( i ).toAttr() );
        if( attribute.isNull() ) continue;

        QString name( attribute.name() );
        QString value( attribute.value() );
        if( name == Xml::ErrorMode ) mode_ = (ErrorMode) value.toUInt();
        else if( name == Xml::ErrorPlus ) plus_ = value.toDouble();
        else if( name == Xml::ErrorMinus ) minus_ = value.toDouble();
        else if( name == Xml::ErrorRelative ) relative_ = value.toUInt();
        else Debug::Throw(0) << "RefPoint::Error::Error - unrecognized attribute: \"" << name << "\"" << endl;
    }

    // parse children elements
    for(QDomNode childNode = element.firstChild(); !childNode.isNull(); childNode = childNode.nextSibling() )
    {
        QDomElement childElement = childNode.toElement();
        if( childElement.isNull() ) continue;
        QString tagName( childElement.tagName() );

        if( tagName == Xml::ErrorGlobalPlus ) globalPlus_ = RefPoint::PointF( childElement );
        else if( tagName == Xml::ErrorGlobalMinus ) globalMinus_ = RefPoint::PointF( childElement );
        else Debug::Throw(0) << "RefPoint::Error::Error - unrecognized child " << childElement.tagName() << endl;
    }

}

//__________________________________________________________________
QDomElement RefPoint::Error::domElement( const QString& name, QDomDocument& parent ) const
{

    Debug::Throw() << "RefPoint::Error::domElement - name: " << name << endl;

    QDomElement out = parent.createElement( name );
    out.setAttribute( Xml::ErrorMode, QString::number(mode_) );
    out.setAttribute( Xml::ErrorPlus, QString::number(plus_) );
    out.setAttribute( Xml::ErrorMinus, QString::number(minus_) );
    out.setAttribute( Xml::ErrorRelative, QString::number(relative_) );

    out.appendChild( globalPlus_.domElement( Xml::ErrorGlobalPlus, parent ) );
    out.appendChild( globalMinus_.domElement( Xml::ErrorGlobalMinus, parent ) );

    return out;
}

//__________________________________________________________________
RefPoint::List::List( const QDomElement& element ):
    Counter( "RefPoint::List" )
{

    Debug::Throw( "RefPoint::List::List (dom).\n" );

    QDomNamedNodeMap attributes( element.attributes() );
    for( int i=0; i<attributes.count(); i++ )
    {
        QDomAttr attribute( attributes.item( i ).toAttr() );
        if( attribute.isNull() ) continue;

        QString name( attribute.name() );
        QString value( attribute.value() );
        if( name == Xml::DrawFlags ) setDrawFlags( (DrawFlags) value.toUInt() );
        else Debug::Throw(0) << "RefPoint::List::List - unrecognized attribute: \"" << name << "\"" << endl;
    }

    // parse children elements
    for(QDomNode childNode = element.firstChild(); !childNode.isNull(); childNode = childNode.nextSibling() )
    {

        QDomElement childElement = childNode.toElement();
        if( childElement.isNull() ) continue;
        QString tagName( childElement.tagName() );

        if( tagName == Xml::Point ) append( RefPoint( childElement ) );
        else Debug::Throw(0) << "RefPoint::List::List - unrecognized child " << childElement.tagName() << endl;

    }

    // some debug output
    Debug::Throw()
        << "RefPoint::List::List -"
        << " drawMarkers: " << drawMarkers()
        << " drawLines: " << drawLines()
        << " drawFlags: " << drawFlags()
        << endl;

}

//__________________________________________________________________
QDomElement RefPoint::List::domElement( const QString& name, QDomDocument& parent ) const
{

    Debug::Throw() << "RefPoint::List::domElement - name: " << name << endl;

    QDomElement out( parent.createElement( name ) );
    out.setAttribute( Xml::DrawFlags, QString::number(drawFlags()) );

    for( const_iterator iter = begin(); iter != end(); ++iter )
    { out.appendChild( iter->domElement( Xml::Point, parent ) ); }

    return out;
}

//__________________________________________________________________
RefPoint::RefPoint( const QDomElement& element ):
    Counter( "RefPoint" ),
    valid_( true )
{
    Debug::Throw( "RefPoint::RefPoint (dom).\n" );

    // parse attributes
    QDomNamedNodeMap attributes( element.attributes() );
    for( int i=0; i<attributes.count(); i++ )
    {
        QDomAttr attribute( attributes.item( i ).toAttr() );
        if( attribute.isNull() ) continue;

        QString name( attribute.name() );
        QString value( attribute.value() );
        if( name == Xml::Valid ) setValid( value.toUInt() );
        else Debug::Throw(0) << "RefPoint::RefPoint - unrecognized attribute: \"" << name << "\"" << endl;

    }

    // parse children elements
    for(QDomNode childNode = element.firstChild(); !childNode.isNull(); childNode = childNode.nextSibling() )
    {
        QDomElement childElement = childNode.toElement();
        if( childElement.isNull() ) continue;
        QString tagName( childElement.tagName() );

        if( tagName == Xml::Global ) setGlobal( PointF( childElement ) );
        else if( tagName == Xml::Local ) setLocal( PointF( childElement ) );
        else if( tagName == Xml::ErrorX ) errorX_ = Error( childElement );
        else if( tagName == Xml::ErrorY ) errorY_ = Error( childElement );
        else if( tagName == Xml::ErrorSystX ) errorSystX_ = Error( childElement );
        else if( tagName == Xml::ErrorSystY ) errorSystY_ = Error( childElement );
        else Debug::Throw(0) << "RefPoint::RefPoint - unrecognized child " << childElement.tagName() << endl;
    }

}

//__________________________________________________________________
QDomElement RefPoint::domElement( const QString& name, QDomDocument& parent ) const
{

    Debug::Throw() << "RefPoint::domElement - name: " << name << endl;

    QDomElement out( parent.createElement( name ) );
    out.setAttribute( Xml::Valid, QString::number( isValid() ) );

    // local position
    out.appendChild( local_.domElement( Xml::Local, parent ) );

    // global position
    out.appendChild( global_.domElement( Xml::Global, parent ) );

    // statistical
    if( hasErrorX() ) out.appendChild( errorX_.domElement( Xml::ErrorX, parent ) );
    if( hasErrorY() ) out.appendChild( errorY_.domElement( Xml::ErrorY, parent ) );

    // systematic
    if( hasErrorSystX() ) out.appendChild( errorSystX_.domElement( Xml::ErrorSystX, parent ) );
    if( hasErrorSystY() ) out.appendChild( errorSystY_.domElement( Xml::ErrorSystY, parent ) );

    return out;

}

//__________________________________________________________________
RefPoint RefPoint::interpolated( const RefPoint& other, qreal x ) const
{

    Q_ASSERT( other.global().x() != global().x() || x == global.x() );
    if( x == global().x() ) return *this;
    else if( x == other.global().x() ) return other;
    else {

        qreal slope = (x - global().x())/(other.global().x() - global().x() );

        RefPoint out( *this );
        out.setGlobal( PointF( x, global().y() + (other.global().y()-global().y())*slope ) );

        // interpolate errors
        if( hasErrorX() && other.hasErrorX() )
        {
            Error error( errorX_ );
            error.mode_ = ErrorAsymetric;
            error.globalPlus_.setX( errorX_.globalPlus_.x() + (other.errorX_.globalPlus_.x() - errorX_.globalPlus_.x())*slope );
            error.globalPlus_.setY( errorX_.globalPlus_.y() + (other.errorX_.globalPlus_.y() - errorX_.globalPlus_.y())*slope );

            error.globalMinus_.setX( errorX_.globalMinus_.x() + (other.errorX_.globalMinus_.x() - errorX_.globalMinus_.x())*slope );
            error.globalMinus_.setY( errorX_.globalMinus_.y() + (other.errorX_.globalMinus_.y() - errorX_.globalMinus_.y())*slope );

            out.errorX_ = error;
        }

        if( hasErrorY() && other.hasErrorY() )
        {
            Error error( errorY_ );
            error.mode_ = ErrorAsymetric;
            error.globalPlus_.setX( errorY_.globalPlus_.x() + (other.errorY_.globalPlus_.x() - errorY_.globalPlus_.x())*slope );
            error.globalPlus_.setY( errorY_.globalPlus_.y() + (other.errorY_.globalPlus_.y() - errorY_.globalPlus_.y())*slope );

            error.globalMinus_.setX( errorY_.globalMinus_.x() + (other.errorY_.globalMinus_.x() - errorY_.globalMinus_.x())*slope );
            error.globalMinus_.setY( errorY_.globalMinus_.y() + (other.errorY_.globalMinus_.y() - errorY_.globalMinus_.y())*slope );

            out.errorY_ = error;
        }

        // interpolate systematic errors
        if( hasErrorSystX() && other.hasErrorSystX() )
        {
            Error error( errorSystX_ );
            error.mode_ = ErrorAsymetric;
            error.globalPlus_.setX( errorSystX_.globalPlus_.x() + (other.errorSystX_.globalPlus_.x() - errorSystX_.globalPlus_.x())*slope );
            error.globalPlus_.setY( errorSystX_.globalPlus_.y() + (other.errorSystX_.globalPlus_.y() - errorSystX_.globalPlus_.y())*slope );

            error.globalMinus_.setX( errorSystX_.globalMinus_.x() + (other.errorSystX_.globalMinus_.x() - errorSystX_.globalMinus_.x())*slope );
            error.globalMinus_.setY( errorSystX_.globalMinus_.y() + (other.errorSystX_.globalMinus_.y() - errorSystX_.globalMinus_.y())*slope );

            out.errorSystX_ = error;
        }

        if( hasErrorSystY() && other.hasErrorSystY() )
        {
            Error error( errorSystY_ );
            error.mode_ = ErrorAsymetric;
            error.globalPlus_.setX( errorSystY_.globalPlus_.x() + (other.errorSystY_.globalPlus_.x() - errorSystY_.globalPlus_.x())*slope );
            error.globalPlus_.setY( errorSystY_.globalPlus_.y() + (other.errorSystY_.globalPlus_.y() - errorSystY_.globalPlus_.y())*slope );

            error.globalMinus_.setX( errorSystY_.globalMinus_.x() + (other.errorSystY_.globalMinus_.x() - errorSystY_.globalMinus_.x())*slope );
            error.globalMinus_.setY( errorSystY_.globalMinus_.y() + (other.errorSystY_.globalMinus_.y() - errorSystY_.globalMinus_.y())*slope );

            out.errorSystY_ = error;
        }

        return out;

    }

}
