// $Id: RefPointErrorDialog.cpp,v 1.15 2014/09/26 08:06:18 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "RefPointErrorDialog.h"
#include "RefPointErrorDialog.moc"

#include "CustomComboBox.h"
#include "GridLayout.h"
#include "XmlOptions.h"

#include <QGroupBox>
#include <QLabel>
#include <QDoubleValidator>

//_____________________________________________________________________
ErrorWidget::ErrorWidget( QWidget* parent ):
    QObject( parent ),
    Counter( "ErrorWidget" )
{
    Debug::Throw( "ErrorWidget::ErrorWidget" );

    // combobox
    combobox_ = new QComboBox( parent );

    // editor
    plusEditor_ = new LineEditor( parent );
    plusEditor_->setAlignment( Qt::AlignRight|Qt::AlignVCenter );
    plusEditor_->setValidator( new QDoubleValidator() );

    // editor
    minusEditor_ = new LineEditor( parent );
    minusEditor_->setAlignment( Qt::AlignRight|Qt::AlignVCenter );
    minusEditor_->setValidator( new QDoubleValidator() );

    // checkbox
    relativeCheckBox_ = new QCheckBox( tr( "Fractional" ), parent );

    // connections and setup
    connect( combobox_, SIGNAL(currentIndexChanged(int)), SLOT(_errorModeChanged(int)) );
    combobox_->addItems( QStringList() << tr( "None" ) << tr( "Symetric" ) << tr( "Asymetric" ) );
}

//_____________________________________________________________________
void ErrorWidget::save( RefPoint::Error& error ) const
{

    error.mode_ = (RefPoint::ErrorMode) combobox_->currentIndex();
    error.relative_ = (RefPoint::ErrorMode) relativeCheckBox_->isChecked();
    error.plus_ = std::abs( plusEditor_->text().toDouble() );
    error.minus_ = (error.mode_ == RefPoint::ErrorSymetric) ? error.plus_ : std::abs( minusEditor_->text().toDouble() );

}

//_____________________________________________________________________
void ErrorWidget::_errorModeChanged( int index )
{
    switch( (RefPoint::ErrorMode) index )
    {
        case RefPoint::NoError:
        plusEditor_->setEnabled( false );
        minusEditor_->setEnabled( false );
        relativeCheckBox_->setEnabled( false );
        break;

        case RefPoint::ErrorSymetric:
        plusEditor_->setEnabled( true );
        minusEditor_->setEnabled( false );
        relativeCheckBox_->setEnabled( true );
        break;

        case RefPoint::ErrorAsymetric:
        plusEditor_->setEnabled( true );
        minusEditor_->setEnabled( true );
        relativeCheckBox_->setEnabled( true );
        break;

        default: break;
    }

}

//_____________________________________________________________________
RefPointErrorDialog::RefPointErrorDialog( QWidget* parent ):
    CustomDialog( parent, OkButton|CancelButton|Separator )
{

    setWindowTitle( "Edit Marker Errors - Graphscan" );
    setOptionName( "REFPOINT_ERROR_DIALOG" );

    GridLayout* gridLayout( new GridLayout() );
    gridLayout->setMaxCount( 2 );
    gridLayout->setMargin(0);
    gridLayout->setSpacing(5);
    mainLayout().addLayout( gridLayout );

    QLabel* label;
    gridLayout->addWidget( label = new QLabel( tr( "Global coordinates:" ), this ) );
    gridLayout->addWidget( globalLabel_ = new QLabel( this ) );
    label->setAlignment( Qt::AlignRight|Qt::AlignVCenter );

    gridLayout->addWidget( label = new QLabel( tr( "Local coordinates:" ), this ) );
    gridLayout->addWidget( localLabel_ = new QLabel( this ) );
    label->setAlignment( Qt::AlignRight|Qt::AlignVCenter );

    QFrame* frame( new QFrame( this ) );
    frame->setFrameStyle( QFrame::HLine );
    mainLayout().addWidget( frame );

    // statistical error
    {
        QGroupBox* groupBox( new QGroupBox( tr( "Statistical errors" ), this ) );
        mainLayout().addWidget( groupBox );

        gridLayout = new GridLayout();
        gridLayout->setMaxCount( 5 );
        gridLayout->setMargin(5);
        gridLayout->setSpacing(5);
        groupBox->setLayout( gridLayout );

        gridLayout->addWidget( label = new QLabel( tr( "Horizontal:" ), this ) );
        label->setAlignment( Qt::AlignRight|Qt::AlignVCenter );
        xErrorWidget_ = new ErrorWidget( groupBox );
        gridLayout->addWidget( &xErrorWidget_->comboBox() );
        gridLayout->addWidget( &xErrorWidget_->plusEditor() );
        gridLayout->addWidget( &xErrorWidget_->minusEditor() );
        gridLayout->addWidget( &xErrorWidget_->relativeCheckBox() );

        gridLayout->addWidget( label = new QLabel( tr( "Vertical:" ), this ) );
        label->setAlignment( Qt::AlignRight|Qt::AlignVCenter );
        yErrorWidget_ = new ErrorWidget( this );
        gridLayout->addWidget( &yErrorWidget_->comboBox() );
        gridLayout->addWidget( &yErrorWidget_->plusEditor() );
        gridLayout->addWidget( &yErrorWidget_->minusEditor() );
        gridLayout->addWidget( &yErrorWidget_->relativeCheckBox() );
    }

    // systematic error
    {
        QGroupBox* groupBox( new QGroupBox( tr( "Systematic errors" ), this ) );
        mainLayout().addWidget( groupBox );

        gridLayout = new GridLayout();
        gridLayout->setMaxCount( 5 );
        gridLayout->setMargin(5);
        gridLayout->setSpacing(5);
        groupBox->setLayout( gridLayout );

        gridLayout->addWidget( label = new QLabel( tr( "Horizontal:" ), this ) );
        label->setAlignment( Qt::AlignRight|Qt::AlignVCenter );
        xErrorSystWidget_ = new ErrorWidget( groupBox );
        gridLayout->addWidget( &xErrorSystWidget_->comboBox() );
        gridLayout->addWidget( &xErrorSystWidget_->plusEditor() );
        gridLayout->addWidget( &xErrorSystWidget_->minusEditor() );
        gridLayout->addWidget( &xErrorSystWidget_->relativeCheckBox() );

        // disable horizontal systematic errors as long as it is not supported
        label->setEnabled( false );
        xErrorSystWidget_->setEnabled( false );

        gridLayout->addWidget( label = new QLabel( tr( "Vertical:" ), this ) );
        label->setAlignment( Qt::AlignRight|Qt::AlignVCenter );
        yErrorSystWidget_ = new ErrorWidget( this );
        gridLayout->addWidget( &yErrorSystWidget_->comboBox() );
        gridLayout->addWidget( &yErrorSystWidget_->plusEditor() );
        gridLayout->addWidget( &yErrorSystWidget_->minusEditor() );
        gridLayout->addWidget( &yErrorSystWidget_->relativeCheckBox() );
    }

}

//_____________________________________________________________________
void RefPointErrorDialog::setPoint( const RefPoint& point )
{

    point_ = point;

    // formats
    globalLabel_->setText( point.global().toString( XmlOptions::get().get<QString>( "GLOBAL_FORMAT" ) ) );
    localLabel_->setText( point.local().toString( XmlOptions::get().get<QString>( "LOCAL_FORMAT" ) ) );

    xErrorWidget_->setError( point.errorX() );
    yErrorWidget_->setError( point.errorY() );

    xErrorSystWidget_->setError( point.errorSystX() );
    yErrorSystWidget_->setError( point.errorSystY() );

}

//_____________________________________________________________________
RefPoint RefPointErrorDialog::point( void ) const
{
    RefPoint out( point_ );

    // errors
    xErrorWidget_->save( out.errorX() );
    yErrorWidget_->save( out.errorY() );

    xErrorSystWidget_->save( out.errorSystX() );
    yErrorSystWidget_->save( out.errorSystY() );

    return out;
}
